/////////////////////////////////////////////////////////////////
//                                                             //
// Tento zdrojov kd je soust distribuce balku program,  //
//     poskytovanch jako doplujc informace ke knize        //
//                                                             //
//                       Java a XML                            //
//                       I. vydn                             //
//                                                             //
//     Pette si, prosm, dkladn upozornn v souboru      //
//                       Cti_me.txt                            //
//        kter je nedlnou soust tto distribuce           //
//                                                             //
//                 (c) Pavel Herout, 2007                      //
//                                                             //
/////////////////////////////////////////////////////////////////

import java.io.*;
import java.util.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;

public class ObezitologieSAX {
  public static final String SOUBOR = "obezitologie-ploche.xml";
  private static HashMap<String, KorenovyHandler> hm;
  private static XMLReader parser; 
    
  static {
    hm = new HashMap<String, KorenovyHandler>();
    hm.put("obezitologie", new ObezitologieHandler("obezitologie", "obezitologie"));
    hm.put("osoba", new OsobaHandler("obezitologie", "osoba"));
    hm.put("jmeno", new JmenoHandler("osoba", "jmeno"));
    hm.put("krestni", new KrestniHandler("jmeno", "krestni"));
    hm.put("prijmeni", new PrijmeniHandler("jmeno", "prijmeni"));
  }
  
  static void zmenContentHandler(String jmeno, Attributes atts) {
//    System.out.println(jmeno);
    KorenovyHandler kh = hm.get(jmeno);
    // nezpracovavany element - vzdy navrat do osoba
    if (kh == null) {
      kh = hm.get("osoba");
    }
    else if (atts != null) {
      kh.zpracujPocatecniZnacku(atts);
    }
    parser.setContentHandler(kh);
  }

  public static void main(String[] args) {
    try {
      // vytvoen parseru
      SAXParserFactory spf = SAXParserFactory.newInstance();
      spf.setValidating(false);
      SAXParser sp = spf.newSAXParser();
      parser = sp.getXMLReader();
      KorenovyHandler kh = (KorenovyHandler) hm.get("obezitologie");

      // nastaven handleru pro parser
      parser.setContentHandler(kh);
      parser.setErrorHandler(new ChybyZjisteneParserem());

      // sputn parseru na soubor
      parser.parse(SOUBOR);
    }
    // zachycen vjimek
    catch (Exception e) {
      e.printStackTrace();
    }
  }
}

class KorenovyHandler extends DefaultHandler {
  protected String nadrizenyJmeno;
  protected String mojeJmeno;
  protected StringBuffer hodnota;
  
  KorenovyHandler(String nadrizenyJmeno, String mojeJmeno) {
    this.nadrizenyJmeno = nadrizenyJmeno;
    this.mojeJmeno = mojeJmeno;
    hodnota = new StringBuffer(100);
  }
  
  protected void nulujHodnotu() {
    hodnota.setLength(0);
  }

  public void zpracujPocatecniZnacku(Attributes atts) {
    nulujHodnotu();
  }

  public void startElement(String namespaceURI, String localName,
                           String qName, Attributes atts) {
    if (qName.equals(mojeJmeno) == true)  {
      return;
    }
    ObezitologieSAX.zmenContentHandler(qName, atts);
  }

  public void endElement(String namespaceURI, String localName,
                         String qName) {
    ObezitologieSAX.zmenContentHandler(nadrizenyJmeno, null);
  }

  public void characters(char[] ch, int start, int length) {
    hodnota.append(ch, start, length);
  }
}

class ObezitologieHandler extends KorenovyHandler {
  ObezitologieHandler(String nadrizenyJmeno, String mojeJmeno) {
    super(nadrizenyJmeno, mojeJmeno);
  }
}

class OsobaHandler extends KorenovyHandler {
  OsobaHandler(String nadrizenyJmeno, String mojeJmeno) {
    super(nadrizenyJmeno, mojeJmeno);
  }
  
  public void zpracujPocatecniZnacku(Attributes atts) {
    nulujHodnotu();
    System.out.println(atts.getValue("role") + ": ");
  }
}

class JmenoHandler extends KorenovyHandler {
  JmenoHandler(String nadrizenyJmeno, String mojeJmeno) {
    super(nadrizenyJmeno, mojeJmeno);
  }
}

class KrestniHandler extends KorenovyHandler {
  KrestniHandler(String nadrizenyJmeno, String mojeJmeno) {
    super(nadrizenyJmeno, mojeJmeno);
  }

  public void endElement(String namespaceURI, String localName,
                         String qName) {
    System.out.println(" krestni = " + hodnota);
    ObezitologieSAX.zmenContentHandler(nadrizenyJmeno, null);
  }
}

class PrijmeniHandler extends KorenovyHandler {
  PrijmeniHandler(String nadrizenyJmeno, String mojeJmeno) {
    super(nadrizenyJmeno, mojeJmeno);
  }

  public void endElement(String namespaceURI, String localName,
                         String qName) {
    System.out.println(" prijmeni = " + hodnota);
    ObezitologieSAX.zmenContentHandler(nadrizenyJmeno, null);
  }
}
